==> gengmei_flutter_plugin-0.0.720.tar.gz: android/src/main/kotlin/com/example/gengmei_flutter_plugin/GengmeiFlutterPlugin.kt <==
package com.example.gengmei_flutter_plugin

import android.Manifest
import android.annotation.TargetApi
import android.app.Activity.RESULT_OK
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.os.Handler
import android.provider.MediaStore
import android.support.v4.content.FileProvider
import android.util.Log
import android.widget.Toast
import com.example.gengmei_flutter_plugin.ImagePlugin.repository.CameraActivity
import com.example.gengmei_flutter_plugin.ImagePlugin.repository.CameraActivity.CAMERA_RESULT
import com.example.gengmei_flutter_plugin.utils.addTo
import com.example.myimagepicker.repository.ImageRespository
import io.flutter.plugin.common.*
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.reactivex.disposables.CompositeDisposable
import zeusees.tracking.FaceTrackingManager
import java.io.File
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

class GengmeiFlutterPlugin : MethodCallHandler {

    val disposable = CompositeDisposable()
    var result: Result? = null

    var isAdded = false;
    private var record: HashMap<String, ArrayList<HashMap<String, Any>>>? = null;
    var nativeImage: File? = null;

    companion object {
        const val IMAGE_PICKER = "scan_image_picker"
        const val NATIVE_CAMERA = "native_camera"
        const val AI_CAMERA = "ai_camera"
        const val DETECT_FACE = "detectPic"
        const val PREMISSION = 10090
        //相机请求码
        private val CAMERA_REQUEST_CODE = 2
        private val CAMERA_REQUEST_CODE_AI = 10012

        //剪裁请求码
        private val CROP_REQUEST_CODE = 3
        lateinit var resign: Registrar;
        var listener: EventChannel.EventSink? = null
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            if (registrar.activity() == null) {
                return
            }
            resign = registrar;
            val gengmeiFlutterPlugin = GengmeiFlutterPlugin();
            val channel = MethodChannel(registrar.messenger(), "gengmei_flutter_plugin")
            channel.setMethodCallHandler(gengmeiFlutterPlugin)
            val eventChannel = EventChannel(registrar.messenger(), "gengmei_flutter_plugin_event")
            eventChannel.setStreamHandler(object : EventChannel.StreamHandler {
                override fun onListen(p0: Any?, p1: EventChannel.EventSink?) {
                    listener = p1;
                }

                override fun onCancel(p0: Any?) {
                    print("ON CANCEL !!!   ");
                    listener = null;
                }
            })
        }
    }

    @TargetApi(Build.VERSION_CODES.M)
    private fun checkPermission(listener: PermissionListener) {
        val activity = resign.activity()
        val writePremission = activity.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)
        val readPremission = activity.checkSelfPermission(Manifest.permission.READ_EXTERNAL_STORAGE)
        val cameraPremission = activity.checkSelfPermission(Manifest.permission.CAMERA);
        if (writePremission != PackageManager.PERMISSION_GRANTED ||
                cameraPremission != PackageManager.PERMISSION_GRANTED ||
                readPremission != PackageManager.PERMISSION_GRANTED) {
            if (activity.shouldShowRequestPermissionRationale(Manifest.permission
                            .WRITE_EXTERNAL_STORAGE)) {
                Toast.makeText(activity.applicationContext, "请开通相关权限，否则无法正常使用本应用！", Toast.LENGTH_SHORT).show()
            }
            if (activity.shouldShowRequestPermissionRationale(Manifest.permission.CAMERA)) {
                Toast.makeText(activity.applicationContext, "请开通相关权限，否则无法正常使用本应用", Toast.LENGTH_SHORT).show()
            }
            if (activity.shouldShowRequestPermissionRationale(Manifest.permission.READ_EXTERNAL_STORAGE)) {
                Toast.makeText(activity.applicationContext, "请开通相关权限，否则无法正常使用本应用", Toast.LENGTH_SHORT).show()
            }
            //申请权限
            activity.requestPermissions(arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE
                    , Manifest.permission.CAMERA, Manifest.permission.READ_EXTERNAL_STORAGE), PREMISSION);
        } else {
//            Toast.makeText(activity.applicationContext, "授权成功！", Toast.LENGTH_SHORT).show();
            listener.OK()
        }
        if (isAdded) {
            return;
        }
        resign.addRequestPermissionsResultListener { id, permissions, grantResults ->
            if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED
                    && grantResults[1] == PackageManager.PERMISSION_GRANTED
                    && grantResults[2] == PackageManager.PERMISSION_GRANTED) {
                listener.OK();
            } else {
                Toast.makeText(activity.applicationContext, "请同意权限", Toast.LENGTH_SHORT).show()
                result?.run {
                    this.error("没有权限！！", "no premission", "没有权限！！");
                }
            }
            true;
        }
        resign.addActivityResultListener { requestCode, resultCode, intent ->
            when (requestCode) {
                CAMERA_REQUEST_CODE -> {
                    if (resultCode == RESULT_OK) {
                        result?.run {
                            if (nativeImage != null) {
                                this.success(nativeImage!!.absolutePath);
                            } else {
                                this.success("");
                            }
                        }
                    } else {
                        result?.run {
                            this.success("")
                        }
                    }
                }
                CAMERA_REQUEST_CODE_AI -> {
                    var stringExtra: String? = "";
                    intent?.run {
                        stringExtra = intent.getStringExtra(CAMERA_RESULT)
                    }
                    result?.run {
                        this.success(stringExtra)
                    }
                }
            }
            true
        };
        isAdded = true;
    }

    private fun gotoNativeCamera() {
        nativeImage = File(resign.activity().getExternalFilesDir(Environment.DIRECTORY_PICTURES), "/GengmeiAi${System.currentTimeMillis()}.jpg");
        val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {   //如果在Android7.0以上,使用FileProvider获取Uri
            intent.setFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
            val contentUri = FileProvider.getUriForFile(resign.activeContext().applicationContext, "com.example.gengmei_flutter_plugin", nativeImage!!);
            intent.putExtra(MediaStore.EXTRA_OUTPUT, contentUri);
        } else {
            intent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(nativeImage));
        }
        resign.activity().startActivityForResult(intent, CAMERA_REQUEST_CODE);
    }

    private fun gotoAiCamera() {
        val intent = Intent(resign.activity(), CameraActivity::class.java)
        resign.activity().startActivityForResult(intent, CAMERA_REQUEST_CODE_AI)
    }


    override fun onMethodCall(call: MethodCall, result: Result) {
        this.result = result
        when (call.method) {
            IMAGE_PICKER -> checkPermission(object : PermissionListener {
                override fun OK() {
                    FaceTrackingManager.getInstance().init(resign.context().applicationContext)
                    ScanImage();
                }
            })
            NATIVE_CAMERA -> checkPermission(object : PermissionListener {
                override fun OK() {
                    gotoNativeCamera()
                }
            })
            AI_CAMERA -> checkPermission(object : PermissionListener {
                override fun OK() {
                    gotoAiCamera()
                }
            })
            DETECT_FACE -> detectImage(call)
            else -> result.notImplemented()
        }
    }

    var detectCount = 0
    private fun detectImageCount(imagePath: String) {
        ImageRespository.getInstance().detectImage(resign.context().applicationContext, imagePath).subscribe({
            if (detectCount > 3) {
                result?.run {
                    this.success("没有检测到人脸")
                }
            } else {
                detectCount++
                if (it == null || it.isEmpty()) {
                    detectImageCount(imagePath);
                } else if (it.size == 1) {
                    result?.run {
                        this.success("success")
                    }
                } else if (it.size > 1) {
                    result?.run {
                        this.success("检测到多张人脸")
                    }
                }

            }


        }, {
            result?.run {
                this.success("检测人脸出错:${it}");
            }
            Toast.makeText(resign.context().applicationContext,
                    it.message, Toast.LENGTH_SHORT).show();
            Log.e("lsy", " ${it}")
        }).addTo(disposable)
    }

    private fun detectImage(call: MethodCall) {
        detectCount = 0
        val imagepath = call.argument<String>("imagepath")!!
        val file = File(imagepath)
        if (!file.exists()) {
            result?.run {
                this.success("这个照片已经被删除了")
            }
        } else {
            if (!FaceTrackingManager.getInstance().isIsInit) {
                result?.run {
                    this.success("模型未初始化")
                }
            } else {
                detectImageCount(imagepath)
            }
        }
    }


    fun ScanImage() {
        ImageRespository.getInstance().scanPhoneImage(resign.context().applicationContext).subscribe(
                {
                    result?.run {
                        record = it
                        Log.e("lsy", "RECORDD $record")
                        this.success(it);
                    }
                    savePreview();
                }, {
            result?.run {
                this.error(it.message, it.message, it);
            }
        }).addTo(disposable);
    }

    fun savePreview() {
        ImageRespository.getInstance().savePreviewImg(resign.context().applicationContext, object : ImageRespository.savePreviewListener {
            override fun onSuccess(data: HashMap<String, ArrayList<HashMap<String, Any>>>) {
                Handler(resign.activeContext().applicationContext.mainLooper).post {
                    listener?.run {
                        this.success(data)
                    }
                }
            }
        });
    }

    interface PermissionListener {
        fun OK()
    }
}

==> gengmei_flutter_plugin-0.0.727.tar.gz: android/src/main/kotlin/com/example/gengmei_flutter_plugin/GengmeiFlutterPlugin.kt <==
package com.example.gengmei_flutter_plugin

import android.Manifest
import android.annotation.TargetApi
import android.app.Activity.RESULT_OK
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.os.Handler
import android.provider.MediaStore
import android.support.v4.content.FileProvider
import android.util.Log
import android.widget.Toast
import com.example.gengmei_flutter_plugin.ImagePlugin.repository.CameraActivity
import com.example.gengmei_flutter_plugin.ImagePlugin.repository.CameraActivity.CAMERA_RESULT
import com.example.gengmei_flutter_plugin.utils.addTo
import com.example.myimagepicker.luban.Luban
import com.example.myimagepicker.repository.ImageRespository
import io.flutter.plugin.common.*
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.reactivex.disposables.CompositeDisposable
import zeusees.tracking.FaceTrackingManager
import java.io.File
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

class GengmeiFlutterPlugin : MethodCallHandler {

    val disposable = CompositeDisposable()
    var result: Result? = null

    var isAdded = false;
    private var record: HashMap<String, ArrayList<HashMap<String, Any>>>? = null;
    var nativeImage: File? = null;

    companion object {
        const val IMAGE_PICKER = "scan_image_picker"
        const val NATIVE_CAMERA = "native_camera"
        const val AI_CAMERA = "ai_camera"
        const val DETECT_FACE = "detectPic"
        const val PREMISSION = 10090
        //相机请求码
        private val CAMERA_REQUEST_CODE = 2
        private val CAMERA_REQUEST_CODE_AI = 10012

        //剪裁请求码
        private val CROP_REQUEST_CODE = 3
        lateinit var resign: Registrar;
        var listener: EventChannel.EventSink? = null
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            if (registrar.activity() == null) {
                return
            }
            resign = registrar;
            val gengmeiFlutterPlugin = GengmeiFlutterPlugin();
            val channel = MethodChannel(registrar.messenger(), "gengmei_flutter_plugin")
            channel.setMethodCallHandler(gengmeiFlutterPlugin)
            val eventChannel = EventChannel(registrar.messenger(), "gengmei_flutter_plugin_event")
            eventChannel.setStreamHandler(object : EventChannel.StreamHandler {
                override fun onListen(p0: Any?, p1: EventChannel.EventSink?) {
                    listener = p1;
                }

                override fun onCancel(p0: Any?) {
                    print("ON CANCEL !!!   ");
                    listener = null;
                }
            })
        }
    }

    @TargetApi(Build.VERSION_CODES.M)
    private fun checkPermission(listener: PermissionListener) {
        val activity = resign.activity()
        val writePremission = activity.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)
        val readPremission = activity.checkSelfPermission(Manifest.permission.READ_EXTERNAL_STORAGE)
        val cameraPremission = activity.checkSelfPermission(Manifest.permission.CAMERA);
        if (writePremission != PackageManager.PERMISSION_GRANTED ||
                cameraPremission != PackageManager.PERMISSION_GRANTED ||
                readPremission != PackageManager.PERMISSION_GRANTED) {
            if (activity.shouldShowRequestPermissionRationale(Manifest.permission
                            .WRITE_EXTERNAL_STORAGE)) {
                Toast.makeText(activity.applicationContext, "请开通相关权限，否则无法正常使用本应用！", Toast.LENGTH_SHORT).show()
            }
            if (activity.shouldShowRequestPermissionRationale(Manifest.permission.CAMERA)) {
                Toast.makeText(activity.applicationContext, "请开通相关权限，否则无法正常使用本应用", Toast.LENGTH_SHORT).show()
            }
            if (activity.shouldShowRequestPermissionRationale(Manifest.permission.READ_EXTERNAL_STORAGE)) {
                Toast.makeText(activity.applicationContext, "请开通相关权限，否则无法正常使用本应用", Toast.LENGTH_SHORT).show()
            }
            //申请权限
            activity.requestPermissions(arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE
                    , Manifest.permission.CAMERA, Manifest.permission.READ_EXTERNAL_STORAGE), PREMISSION);
        } else {
//            Toast.makeText(activity.applicationContext, "授权成功！", Toast.LENGTH_SHORT).show();
            listener.OK()
        }
        if (isAdded) {
            return;
        }
        resign.addRequestPermissionsResultListener { id, permissions, grantResults ->
            if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED
                    && grantResults[1] == PackageManager.PERMISSION_GRANTED
                    && grantResults[2] == PackageManager.PERMISSION_GRANTED) {
                listener.OK();
            } else {
                Toast.makeText(activity.applicationContext, "请同意权限", Toast.LENGTH_SHORT).show()
                result?.run {
                    this.error("没有权限！！", "no premission", "没有权限！！");
                }
            }
            true;
        }
        resign.addActivityResultListener { requestCode, resultCode, intent ->
            when (requestCode) {
                CAMERA_REQUEST_CODE -> {
                    if (resultCode == RESULT_OK) {
                        result?.run {
                            if (nativeImage != null) {
                                this.success(nativeImage!!.absolutePath);
                            } else {
                                this.success("");
                            }
                        }
                    } else {
                        result?.run {
                            this.success("")
                        }
                    }
                }
                CAMERA_REQUEST_CODE_AI -> {
                    var stringExtra: String? = "";
                    intent?.run {
                        stringExtra = intent.getStringExtra(CAMERA_RESULT)
                    }
                    result?.run {
                        this.success(stringExtra)
                    }
                }
            }
            true
        };
        isAdded = true;
    }

    private fun gotoNativeCamera() {
        nativeImage = File(resign.activity().getExternalFilesDir(Environment.DIRECTORY_PICTURES), "/GengmeiAi${System.currentTimeMillis()}.jpg");
        val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {   //如果在Android7.0以上,使用FileProvider获取Uri
            intent.setFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
            val contentUri = FileProvider.getUriForFile(resign.activeContext().applicationContext, "com.example.gengmei_flutter_plugin", nativeImage!!);
            intent.putExtra(MediaStore.EXTRA_OUTPUT, contentUri);
        } else {
            intent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(nativeImage));
        }
        resign.activity().startActivityForResult(intent, CAMERA_REQUEST_CODE);
    }

    private fun gotoAiCamera() {
        val intent = Intent(resign.activity(), CameraActivity::class.java)
        resign.activity().startActivityForResult(intent, CAMERA_REQUEST_CODE_AI)
    }


    override fun onMethodCall(call: MethodCall, result: Result) {
        this.result = result
        when (call.method) {
            IMAGE_PICKER -> checkPermission(object : PermissionListener {
                override fun OK() {
                    FaceTrackingManager.getInstance().init(resign.context().applicationContext)
                    ScanImage();
                }
            })
            NATIVE_CAMERA -> checkPermission(object : PermissionListener {
                override fun OK() {
                    gotoNativeCamera()
                }
            })
            AI_CAMERA -> checkPermission(object : PermissionListener {
                override fun OK() {
                    gotoAiCamera()
                }
            })
            DETECT_FACE -> detectImage(call)
            else -> result.notImplemented()
        }
    }

    var detectCount = 0
    private fun detectImageCount(imagePath: String) {
        ImageRespository.getInstance().detectImage(resign.context().applicationContext, imagePath).subscribe({
            if (detectCount > 3) {
                result?.run {
                    this.success("没有检测到人脸")
                }
            } else {
                detectCount++
                if (it == null || it.isEmpty()) {
                    detectImageCount(imagePath);
                } else if (it.size == 1) {
                    result?.run {
                        this.success("success")
                    }
                } else if (it.size > 1) {
                    result?.run {
                        this.success("检测到多张人脸")
                    }
                }

            }


        }, {
            result?.run {
                this.success("检测人脸出错:${it}");
            }
            Toast.makeText(resign.context().applicationContext,
                    it.message, Toast.LENGTH_SHORT).show();
            Log.e("lsy", " ${it}")
        }).addTo(disposable)
    }

    private fun detectImage(call: MethodCall) {
        detectCount = 0
        val imagepath = call.argument<String>("imagepath")!!
        val file = File(imagepath)
        if (!file.exists()) {
            result?.run {
                this.success("这个照片已经被删除了")
            }
        } else {
            if (!FaceTrackingManager.getInstance().isIsInit) {
                result?.run {
                    this.success("模型未初始化")
                }
            } else {
                Thread {
                    val get = Luban.with(resign.context().applicationContext).quality(100).get(imagepath);
                    detectImageCount(get.absolutePath)
                }.start()
            }
        }
    }


    fun ScanImage() {
        ImageRespository.getInstance().scanPhoneImage(resign.context().applicationContext).subscribe(
                {
                    result?.run {
                        record = it
                        Log.e("lsy", "RECORDD $record")
                        this.success(it);
                        savePreview();
                    }
                }, {
            result?.run {
                this.error(it.message, it.message, it);
            }
        }).addTo(disposable);
    }

    fun savePreview() {
        ImageRespository.getInstance().savePreviewImg(resign.context().applicationContext, object : ImageRespository.savePreviewListener {
            override fun onSuccess(data: HashMap<String, ArrayList<HashMap<String, Any>>>) {
                Handler(resign.activeContext().applicationContext.mainLooper).post {
                    listener?.run {
                        this.success(data)
                    }
                }
            }
        });
    }

    interface PermissionListener {
        fun OK()
    }
}
